"""
テストケースに登録された添付ファイルを取得します。

使い方：
1. TestRail の [管理] > [サイト設定] > [API] タブで 'API の有効化' チェックボックスを有効化してください。
2. ログイン情報をご利用環境に合わせて変更してください。
3. 実行時引数を3つ指定してください。
   ・第一引数：プロジェクトID
   ・第二引数：テストスイートID
   ・第三引数：ファイルダウンロード先フォルダーパス (事前に用意してください。)
    実行例： python.exe get_attachment_case.py 5 13 C:\\sample
    ↑プロジェクト ID が 5 でテストスイート ID 13 のテストケースに添付されたファイルを C:\\sample にダウンロードします。
4. 指定したフォルダーパスに 'テストスイートID\テストケースID' のフォルダーを作成してファイルをダウンロードします。

出力されるファイル：
1. テストケースに添付されたファイル
2. テストケースのテキストフィールドに埋め込まれた画像ファイル
3. テストケース(result.json)

その他：
1. 取得される添付ファイルのファイル名は以下のルールに従って出力されます。
   TestRailが自動で採番したIDとファイル名を'_'で連結しファイル名としてダウンロードします。
   どこに埋め込まれた画像であるかは、出力された「3. テストケース」の内容を確認してください。
2. 全てのテストケースの添付ファイルを取得します。
   優先度や、作成日付で絞込みを行う場合 'get_cases' のリクエストフィルターを使用してください。
   'get_results_for_run' の使用方法は以下のマニュアルを参照してください。
   https://docs.testrail.techmatrix.jp/api/reference/cases/#get_cases

"""

import sys
import os
import re
import json
import codecs
from testrail import *
from testrail_webclient import *

# 実行時引数の配列
# 第一引数：テストランID
# 第二引数：フォルダーパス
args = sys.argv

# TestRail ログイン情報 (★ ご利用環境に合わせて変更してください。)
testrail_url = "http://localhost:8000/"
user         = "user@techmatrix.co.jp"
password     = "password"

client = APIClient(testrail_url)
client.user = user
client.password = password

# テストスイートに含まれるテストケース情報を取得 (★ 'get_cases' のリクエストフィルター)
response_suite_cases = client.send_get('get_cases/' + args[1] + '&suite_id=' + args[2])

if 'offset' in response_suite_cases:
    dict_suite_cases = response_suite_cases['cases']
    mode = "new"
else:
    dict_suite_cases = response_suite_cases
    mode = "old"

print(str(len(dict_suite_cases)) + " 件のテストケースがあります。")

# テストケースの数分繰り返す
for i, one_case in enumerate(dict_suite_cases):
    # ケースID
    case_id = one_case.get('id')

    print(str((i + 1)) + " 件目(" + str(len(dict_suite_cases)) + " 件中)を処理しています。ケースID " + str(case_id))

    # ダウンロード先フォルダー
    download_folder = args[3] + '\\' + args[2] + '\\' + str(case_id)

    # テストケース情報をJSON ⇒ 辞書型 ⇒ 文字列型に変換
    one_case_dict = json.dumps(one_case)
    one_case_str = json.dumps(one_case_dict)

    # 埋め込み画像情報を取得
    pattern = r'index.php\?/attachments/get/([\w-]+)'
    embedded_image_list = re.findall(pattern, one_case_str)

    # 埋め込み画像のID取得用
    embedded_image_list_id = []

    # 埋め込む画像数分繰り返す
    for one_embedded_image in embedded_image_list:

        # フォルダーがある場合は作成しない
        if not os.path.isdir(download_folder):
            os.makedirs(download_folder)

        # 埋め込み画像の ID
        embedded_image_id = str(re.sub(r'index.php\?/attachments/get/|\)', '', one_embedded_image))

        # 埋め込み画像の ID をリストに追加
        embedded_image_list_id.append(embedded_image_id)

        # ファイルをダウンロード
        webclient = WebClient(testrail_url, user, password)
        filename, content = webclient.get_attachement(embedded_image_id)

        # 添付ファイル保存
        with open(download_folder + '/' + embedded_image_id + '_' + filename, mode='wb') as f:
            f.write(content)

    # テストケースの添付ファイルのリストを取得 (リストには複数回のテスト結果に添付されたファイル情報が含まれる)
    response = client.send_get('get_attachments_for_case/' + str(case_id))
    if mode == "new":
        list_attachment = response['attachments']
    else:
        list_attachment = response

    # 添付ファイル数分繰り返す
    for one_attachment in list_attachment:

        # ケースIDが同じ場合だけ取得する
        if mode == "new":
            target_id = str(one_attachment.get('entity_id'))
        else:
            target_id = str(one_attachment.get('case_id'))

        if (str(case_id) == target_id):

            # フォルダーがある場合は作成しない
            if not os.path.isdir(download_folder):
                os.makedirs(download_folder)

            # ファイルID
            file_id = str(one_attachment.get('id'))

            # ファイル ID が埋め込み画像の ID と一致する場合は取得しない
            if not file_id in embedded_image_list_id:
                # ファイルをダウンロード
                webclient = WebClient(testrail_url, user, password)
                filename, content = webclient.get_attachement(file_id)

                # 添付ファイル保存
                if filename != "":
                    with open(download_folder + '/' + file_id + '_' + filename, mode='wb') as f:
                        f.write(content)

    # フォルダーがある場合はテストケース情報をファイル出力する
    if os.path.isdir(download_folder):
        # テストケース情報をファイル出力
        result_file_path = download_folder + '\\result.json'

        with codecs.open(result_file_path, 'w', "utf-8") as f:
            json.dump(one_case, f, ensure_ascii=False, indent=4)
